@@ -168,6 +168,12 @@ static const struct bdc bdcms[] =
 		.argtype = ARG_LONG,
 		.argval = -1,
 		.help = N_("get filesystem readahead")
+	},{
+		IOCTL_ENTRY(BLKGETDISKSEQ),
+		.name = "--getdiskseq",
+		.argtype = ARG_ULLONG,
+		.argval = -1,
+		.help = N_("get disk sequence number")
 	},{
 		IOCTL_ENTRY(BLKFLSBUF),
 		.name = "--flushbufs",
