@@ -149,7 +149,7 @@ static FILE *report_stats_file;
 static int num_running;
 static int max_running;
 
-static volatile int cancel_requested;
+static volatile sig_atomic_t cancel_requested;
 static int kill_sent;
 static char *fstype;
 static struct fsck_instance *instance_list;
@@ -730,6 +730,8 @@ static int kill_all(int signum)
 	for (inst = instance_list; inst; inst = inst->next) {
 		if (inst->flags & FLAG_DONE)
 			continue;
+		if (inst->pid <= 0)
+			continue;
 		kill(inst->pid, signum);
 		n++;
 	}
@@ -1420,7 +1422,7 @@ static void __attribute__((__noreturn__)) usage(void)
 
 static void signal_cancel(int sig __attribute__((__unused__)))
 {
-	cancel_requested++;
+	cancel_requested = 1;
 }
 
 static void parse_argv(int argc, char *argv[])
@@ -1628,6 +1630,16 @@ int main(int argc, char *argv[])
 	mnt_init_debug(0);		/* init libmount debug mask */
 	mntcache = mnt_new_cache();	/* no fatal error if failed */
 
+	if (mntcache)
+		/* Force libblkid to accept also filesystems with bad
+		 * checksums. This feature is helpful for "fsck /dev/foo," but
+		 * if it evaluates LABEL/UUIDs from fstab, then libmount may
+		 * use cached data from udevd and udev accepts only properly
+		 * detected filesystems.
+		 */
+		mnt_cache_set_sbprobe(mntcache, BLKID_SUBLKS_BADCSUM);
+
+
 	parse_argv(argc, argv);
 
 	if (!notitle)
