@@ -1,4 +1,3 @@
-
 /*
  * No copyright is claimed.  This code is in the public domain; do with
  * it what you wish.
@@ -59,6 +58,7 @@ DEF_GUID("993D8D3D-F80E-4225-855A-9DAF8ED7EA97", N_("Linux root (IA-64)")),
 DEF_GUID("77055800-792C-4F94-B39A-98C91B762BB6", N_("Linux root (LoongArch-64)")),
 DEF_GUID("37C58C8A-D913-4156-A25F-48B1B64E07F0", N_("Linux root (MIPS-32 LE)")),
 DEF_GUID("700BDA43-7A34-4507-B179-EEB93D7A7CA3", N_("Linux root (MIPS-64 LE)")),
+DEF_GUID("1AACDB3B-5444-4138-BD9E-E5C2239B2346", N_("Linux root (HPPA/PARISC)")),
 DEF_GUID("1DE3F1EF-FA98-47B5-8DCD-4A860A654D78", N_("Linux root (PPC)")),
 DEF_GUID("912ADE1D-A839-4913-8964-A10EEE08FBD2", N_("Linux root (PPC64)")),
 DEF_GUID("C31C45E6-3F39-412E-80FB-4809C4980599", N_("Linux root (PPC64LE)")),
@@ -83,6 +83,7 @@ DEF_GUID("4301D2A6-4E3B-4B2A-BB94-9E0B2C4225EA", N_("Linux /usr (IA-64)")),
 DEF_GUID("E611C702-575C-4CBE-9A46-434FA0BF7E3F", N_("Linux /usr (LoongArch-64)")),
 DEF_GUID("0F4868E9-9952-4706-979F-3ED3A473E947", N_("Linux /usr (MIPS-32 LE)")),
 DEF_GUID("C97C1F32-BA06-40B4-9F22-236061B08AA8", N_("Linux /usr (MIPS-64 LE)")),
+DEF_GUID("DC4A4480-6917-4262-A4EC-DB9384949F25", N_("Linux /usr (HPPA/PARISC)")),
 DEF_GUID("7D14FEC5-CC71-415D-9D6C-06BF0B3C3EAF", N_("Linux /usr (PPC)")),
 DEF_GUID("2C9739E2-F068-46B3-9FD0-01C5A9AFBCCA", N_("Linux /usr (PPC64)")),
 DEF_GUID("15BB03AF-77E7-4D4A-B12B-C0D084F7491C", N_("Linux /usr (PPC64LE)")),
@@ -101,6 +102,7 @@ DEF_GUID("86ED10D5-B607-45BB-8957-D350F23D0571", N_("Linux root verity (IA-64)")
 DEF_GUID("F3393B22-E9AF-4613-A948-9D3BFBD0C535", N_("Linux root verity (LoongArch-64)")),
 DEF_GUID("D7D150D2-2A04-4A33-8F12-16651205FF7B", N_("Linux root verity (MIPS-32 LE)")),
 DEF_GUID("16B417F8-3E06-4F57-8DD2-9B5232F41AA6", N_("Linux root verity (MIPS-64 LE)")),
+DEF_GUID("D212A430-FBC5-49F9-A983-A7FEEF2B8D0E", N_("Linux root verity (HPPA/PARISC)")),
 DEF_GUID("98CFE649-1588-46DC-B2F0-ADD147424925", N_("Linux root verity (PPC)")),
 DEF_GUID("9225A9A3-3C19-4D89-B4F6-EEFF88F17631", N_("Linux root verity (PPC64)")),
 DEF_GUID("906BD944-4589-4AAE-A4E4-DD983917446A", N_("Linux root verity (PPC64LE)")),
@@ -119,6 +121,7 @@ DEF_GUID("6A491E03-3BE7-4545-8E38-83320E0EA880", N_("Linux /usr verity (IA-64)")
 DEF_GUID("F46B2C26-59AE-48F0-9106-C50ED47F673D", N_("Linux /usr verity (LoongArch-64)")),
 DEF_GUID("46B98D8D-B55C-4E8F-AAB3-37FCA7F80752", N_("Linux /usr verity (MIPS-32 LE)")),
 DEF_GUID("3C3D61FE-B5F3-414D-BB71-8739A694A4EF", N_("Linux /usr verity (MIPS-64 LE)")),
+DEF_GUID("5843D618-EC37-48D7-9F12-CEA8E08768B2", N_("Linux /usr verity (HPPA/PARISC)")),
 DEF_GUID("DF765D00-270E-49E5-BC75-F47BB2118B09", N_("Linux /usr verity (PPC)")),
 DEF_GUID("BDB528A5-A259-475F-A87D-DA53FA736A07", N_("Linux /usr verity (PPC64)")),
 DEF_GUID("EE2B9983-21E8-4153-86D9-B6901A54D1CE", N_("Linux /usr verity (PPC64LE)")),
@@ -137,6 +140,7 @@ DEF_GUID("E98B36EE-32BA-4882-9B12-0CE14655F46A", N_("Linux root verity sign. (IA
 DEF_GUID("5AFB67EB-ECC8-4F85-AE8E-AC1E7C50E7D0", N_("Linux root verity sign. (LoongArch-64)")),
 DEF_GUID("C919CC1F-4456-4EFF-918C-F75E94525CA5", N_("Linux root verity sign. (MIPS-32 LE)")),
 DEF_GUID("904E58EF-5C65-4A31-9C57-6AF5FC7C5DE7", N_("Linux root verity sign. (MIPS-64 LE)")),
+DEF_GUID("15DE6170-65D3-431C-916E-B0DCD8393F25", N_("Linux root verity sign. (HPPA/PARISC)")),
 DEF_GUID("1B31B5AA-ADD9-463A-B2ED-BD467FC857E7", N_("Linux root verity sign. (PPC)")),
 DEF_GUID("F5E2C20C-45B2-4FFA-BCE9-2A60737E1AAF", N_("Linux root verity sign. (PPC64)")),
 DEF_GUID("D4A236E7-E873-4C07-BF1D-BF6CF7F1C3C6", N_("Linux root verity sign. (PPC64LE)")),
@@ -155,6 +159,7 @@ DEF_GUID("8DE58BC2-2A43-460D-B14E-A76E4A17B47F", N_("Linux /usr verity sign. (IA
 DEF_GUID("B024F315-D330-444C-8461-44BBDE524E99", N_("Linux /usr verity sign. (LoongArch-64)")),
 DEF_GUID("3E23CA0B-A4BC-4B4E-8087-5AB6A26AA8A9", N_("Linux /usr verity sign. (MIPS-32 LE)")),
 DEF_GUID("F2C2C7EE-ADCC-4351-B5C6-EE9816B66E16", N_("Linux /usr verity sign. (MIPS-64 LE)")),
+DEF_GUID("450DD7D1-3224-45EC-9CF2-A43A346D71EE", N_("Linux /usr verity sign. (HPPA/PARISC)")),
 DEF_GUID("7007891D-D371-4A80-86A4-5CB875B9302E", N_("Linux /usr verity sign. (PPC)")),
 DEF_GUID("0B888863-D7F8-4D9E-9766-239FCE4D58AF", N_("Linux /usr verity sign. (PPC64)")),
 DEF_GUID("C8BFBD1E-268E-4521-8BBA-BF314C399557", N_("Linux /usr verity sign. (PPC64LE)")),
