@@ -12,6 +12,7 @@
 #include <errno.h>
 #include <sys/stat.h>
 #include <string.h>
+#include <strings.h>
 #include <assert.h>
 
 #include "c.h"
@@ -326,13 +327,16 @@ int ul_strtos64(const char *str, int64_t *num, int base)
 {
 	char *end = NULL;
 
-	errno = 0;
 	if (str == NULL || *str == '\0')
-		return -EINVAL;
+		return -(errno = EINVAL);
+
+	errno = 0;
 	*num = (int64_t) strtoimax(str, &end, base);
 
-	if (errno || str == end || (end && *end))
-		return -EINVAL;
+	if (errno != 0)
+		return -errno;
+	if (str == end || (end && *end))
+		return -(errno = EINVAL);
 	return 0;
 }
 
@@ -341,13 +345,13 @@ int ul_strtou64(const char *str, uint64_t *num, int base)
 	char *end = NULL;
 	int64_t tmp;
 
-	errno = 0;
 	if (str == NULL || *str == '\0')
-		return -EINVAL;
+		return -(errno = EINVAL);
 
 	/* we need to ignore negative numbers, note that for invalid negative
 	 * number strtoimax() returns negative number too, so we do not
 	 * need to check errno here */
+	errno = 0;
 	tmp = (int64_t) strtoimax(str, &end, base);
 	if (tmp < 0)
 		errno = ERANGE;
@@ -356,8 +360,10 @@ int ul_strtou64(const char *str, uint64_t *num, int base)
 		*num = strtoumax(str, &end, base);
 	}
 
-	if (errno || str == end || (end && *end))
-		return -EINVAL;
+	if (errno != 0)
+		return -errno;
+	if (str == end || (end && *end))
+		return -(errno = EINVAL);
 	return 0;
 }
 
@@ -388,10 +394,10 @@ int ul_strtou32(const char *str, uint32_t *num, int base)
 }
 
 /*
- * Covert strings to numbers in defined range and print message on error.
+ * Convert strings to numbers in defined range and print message on error.
  *
  * These functions are used when we read input from users (getopt() etc.). It's
- * better to consolidate the code and keep it all based on 64-bit numbers then
+ * better to consolidate the code and keep it all based on 64-bit numbers than
  * implement it for 32 and 16-bit numbers too.
  */
 int64_t str2num_or_err(const char *str, int base, const char *errmesg,
@@ -471,48 +477,6 @@ err:
 	errx(STRTOXX_EXIT_CODE, "%s: '%s'", errmesg, str);
 }
 
-long strtol_or_err(const char *str, const char *errmesg)
-{
-	long num;
-	char *end = NULL;
-
-	errno = 0;
-	if (str == NULL || *str == '\0')
-		goto err;
-	num = strtol(str, &end, 10);
-
-	if (errno || str == end || (end && *end))
-		goto err;
-
-	return num;
-err:
-	if (errno == ERANGE)
-		err(STRTOXX_EXIT_CODE, "%s: '%s'", errmesg, str);
-
-	errx(STRTOXX_EXIT_CODE, "%s: '%s'", errmesg, str);
-}
-
-unsigned long strtoul_or_err(const char *str, const char *errmesg)
-{
-	unsigned long num;
-	char *end = NULL;
-
-	errno = 0;
-	if (str == NULL || *str == '\0')
-		goto err;
-	num = strtoul(str, &end, 10);
-
-	if (errno || str == end || (end && *end))
-		goto err;
-
-	return num;
-err:
-	if (errno == ERANGE)
-		err(STRTOXX_EXIT_CODE, "%s: '%s'", errmesg, str);
-
-	errx(STRTOXX_EXIT_CODE, "%s: '%s'", errmesg, str);
-}
-
 uintmax_t strtosize_or_err(const char *str, const char *errmesg)
 {
 	uintmax_t num;
@@ -536,6 +500,15 @@ void strtotimeval_or_err(const char *str, struct timeval *tv, const char *errmes
 	tv->tv_usec = (suseconds_t)((user_input - tv->tv_sec) * 1000000);
 }
 
+void strtotimespec_or_err(const char *str, struct timespec *ts, const char *errmesg)
+{
+	long double user_input;
+
+	user_input = strtold_or_err(str, errmesg);
+	ts->tv_sec = (time_t) user_input;
+	ts->tv_nsec = (long)((user_input - ts->tv_sec) * 1000000000);
+}
+
 time_t strtotime_or_err(const char *str, const char *errmesg)
 {
 	int64_t user_input;
@@ -761,11 +734,16 @@ int string_add_to_idarray(const char *list, int ary[], size_t arysz,
  * as a position in the 'ary' bit array. It means that the 'id' has to be in
  * range <0..N> where N < sizeof(ary) * NBBY.
  *
+ * If allow_range is enabled:
+ * An item ending in '+' also sets all bits in <0..N>.
+ * An item beginning with '+' also sets all bits in <N..allow_minus>.
+ *
  * Returns: 0 on success, <0 on error.
  */
 int string_to_bitarray(const char *list,
 		     char *ary,
-		     int (*name2bit)(const char *, size_t))
+		     int (*name2bit)(const char *, size_t),
+		     size_t allow_range)
 {
 	const char *begin = NULL, *p;
 
@@ -774,7 +752,7 @@ int string_to_bitarray(const char *list,
 
 	for (p = list; p && *p; p++) {
 		const char *end = NULL;
-		int bit;
+		int bit, set_lower = 0, set_higher = 0;
 
 		if (!begin)
 			begin = p;		/* begin of the level name */
@@ -786,11 +764,26 @@ int string_to_bitarray(const char *list,
 			continue;
 		if (end <= begin)
 			return -1;
+		if (allow_range) {
+			if (*(end - 1) == '+') {
+				end--;
+				set_lower = 1;
+			} else if (*begin == '+') {
+				begin++;
+				set_higher = 1;
+			}
+		}
 
 		bit = name2bit(begin, end - begin);
 		if (bit < 0)
 			return bit;
 		setbit(ary, bit);
+		if (set_lower)
+			while (--bit >= 0)
+				setbit(ary, bit);
+		else if (set_higher)
+			while (++bit < (int) allow_range)
+				setbit(ary, bit);
 		begin = NULL;
 		if (end && !*end)
 			break;
@@ -936,7 +929,8 @@ int streq_paths(const char *a, const char *b)
 	return 0;
 }
 
-char *strnappend(const char *s, const char *suffix, size_t b)
+/* concatenate two strings to a new string, the size of the second string is limited by @b */
+char *strnconcat(const char *s, const char *suffix, size_t b)
 {
         size_t a;
         char *r;
@@ -966,12 +960,14 @@ char *strnappend(const char *s, const char *suffix, size_t b)
         return r;
 }
 
-char *strappend(const char *s, const char *suffix)
+/* concatenate two strings to a new string */
+char *strconcat(const char *s, const char *suffix)
 {
-        return strnappend(s, suffix, suffix ? strlen(suffix) : 0);
+        return strnconcat(s, suffix, suffix ? strlen(suffix) : 0);
 }
 
-char *strfappend(const char *s, const char *format, ...)
+/* concatenate @s and string defined by @format to a new string */
+char *strfconcat(const char *s, const char *format, ...)
 {
 	va_list ap;
 	char *val, *res;
@@ -984,11 +980,36 @@ char *strfappend(const char *s, const char *format, ...)
 	if (sz < 0)
 		return NULL;
 
-	res = strnappend(s, val, sz);
+	res = strnconcat(s, val, sz);
 	free(val);
 	return res;
 }
 
+int strappend(char **a, const char *b)
+{
+	size_t al, bl;
+	char *tmp;
+
+	if (!a)
+		return -EINVAL;
+	if (!b || !*b)
+		return 0;
+	if (!*a) {
+		*a = strdup(b);
+		return !*a ? -ENOMEM : 0;
+	}
+
+	al = strlen(*a);
+	bl = strlen(b);
+
+	tmp = realloc(*a, al + bl + 1);
+	if (!tmp)
+		return -ENOMEM;
+	*a = tmp;
+	memcpy((*a) + al, b, bl + 1);
+	return 0;
+}
+
 static size_t strcspn_escaped(const char *s, const char *reject)
 {
         int escaped = 0;
@@ -1007,6 +1028,37 @@ static size_t strcspn_escaped(const char *s, const char *reject)
         return n - escaped;
 }
 
+/*
+ * Like strchr() but ignores @c if escaped by '\', '\\' is interpreted like '\'.
+ *
+ * For example for @c='X':
+ *
+ *      "abcdXefgXh"    --> "XefgXh"
+ *	"abcd\XefgXh"   --> "Xh"
+ *	"abcd\\XefgXh"  --> "XefgXh"
+ *	"abcd\\\XefgXh" --> "Xh"
+ *	"abcd\Xefg\Xh"  --> (null)
+ *
+ *	"abcd\\XefgXh"  --> "\XefgXh"   for @c='\\'
+ */
+char *ul_strchr_escaped(const char *s, int c)
+{
+	char *p;
+	int esc = 0;
+
+	for (p = (char *) s; p && *p; p++) {
+		if (!esc && *p == '\\') {
+			esc = 1;
+			continue;
+		}
+		if (*p == c && (!esc || c == '\\'))
+			return p;
+		esc = 0;
+	}
+
+	return NULL;
+}
+
 /* Split a string into words. */
 const char *split(const char **state, size_t *l, const char *separator, int quoted)
 {
@@ -1095,7 +1147,80 @@ int ul_stralnumcmp(const char *p1, const char *p2)
 	return c1 - c2;
 }
 
+/*
+ * Parses the first option from @optstr. The @optstr pointer is set to the beginning
+ * of the next option. The options string looks like 'aaa,bbb=data,foo,bar="xxx"'.
+ *
+ * Note this function is used by libmount to parse mount options. Be careful when modify.
+ *
+ * Returns -EINVAL on parse error, 1 at the end of optstr and 0 on success.
+ */
+int ul_optstr_next(char **optstr, char **name, size_t *namesz,
+		   char **value, size_t *valsz)
+{
+	int open_quote = 0;
+	char *start = NULL, *stop = NULL, *p, *sep = NULL;
+	char *optstr0;
+
+	assert(optstr);
+	assert(*optstr);
+
+	optstr0 = *optstr;
+
+	if (name)
+		*name = NULL;
+	if (namesz)
+		*namesz = 0;
+	if (value)
+		*value = NULL;
+	if (valsz)
+		*valsz = 0;
+
+	/* trim leading commas as to not invalidate option
+	 * strings with multiple consecutive commas */
+	while (optstr0 && *optstr0 == ',')
+		optstr0++;
+
+	for (p = optstr0; p && *p; p++) {
+		if (!start)
+			start = p;		/* beginning of the option item */
+		if (*p == '"')
+			open_quote ^= 1;	/* reverse the status */
+		if (open_quote)
+			continue;		/* still in quoted block */
+		if (!sep && p > start && *p == '=')
+			sep = p;		/* name and value separator */
+		if (*p == ',')
+			stop = p;		/* terminate the option item */
+		else if (*(p + 1) == '\0')
+			stop = p + 1;		/* end of optstr */
+		if (!start || !stop)
+			continue;
+		if (stop <= start)
+			return -EINVAL;
+
+		if (name)
+			*name = start;
+		if (namesz)
+			*namesz = sep ? sep - start : stop - start;
+		*optstr = *stop ? stop + 1 : stop;
+
+		if (sep) {
+			if (value)
+				*value = sep + 1;
+			if (valsz)
+				*valsz = stop - sep - 1;
+		}
+		return 0;
+	}
+
+	return 1;				/* end of optstr */
+}
+
 #ifdef TEST_PROGRAM_STRUTILS
+
+#include "cctype.h"
+
 struct testS {
 	char *name;
 	char *value;
@@ -1166,17 +1291,52 @@ static int test_strutils_cmp_paths(int argc, char *argv[])
 
 static int test_strutils_normalize(int argc, char *argv[])
 {
-	unsigned char *str;
-	size_t sz;
+	unsigned char *src, *dst, *org;
+	size_t sz, len;
 
 	if (argc < 2)
 		return EXIT_FAILURE;
 
-	str = (unsigned char *) strdup(argv[1]);
-	sz = normalize_whitespace(str);
+	org = (unsigned char *) strdup(argv[1]);
+	src = (unsigned char *) strdup((char *) org);
+	len = strlen((char *) src);
+	dst = malloc(len + 1);
 
-	printf("'%s' --> '%s' [sz=%zu]\n", argv[1], str, sz);
-	free(str);
+	if (!org || !src || !dst)
+		goto done;
+
+	/* two buffers */
+	sz = __normalize_whitespace(src, len, dst, len + 1);
+	printf("1: '%s' --> '%s' [sz=%zu]\n", src, dst, sz);
+
+	/* one buffer */
+	sz = normalize_whitespace(src);
+	printf("2: '%s' --> '%s' [sz=%zu]\n", org, src, sz);
+
+done:
+	free(src);
+	free(dst);
+	free(org);
+
+	return EXIT_SUCCESS;
+}
+
+static int test_strutils_cstrcasecmp(int argc, char *argv[])
+{
+	char *a, *b;
+
+	if (argc < 3)
+		return EXIT_FAILURE;
+
+	a = argv[1];
+	b = argv[2];
+
+	if (!a || !b)
+		return EXIT_FAILURE;
+
+	printf("cmp    '%s' '%s' = %d\n", a, b, strcasecmp(a, b));
+	printf("c_cmp  '%s' '%s' = %d\n", a, b, c_strcasecmp(a, b));
+	printf("c_ncmp '%s' '%s' = %d\n", a, b, c_strncasecmp(a, b, strlen(a)));
 
 	return EXIT_SUCCESS;
 }
@@ -1196,10 +1356,13 @@ int main(int argc, char *argv[])
 		printf("%s\n", ul_stralnumcmp(argv[2], argv[3]) == 0 ?
 				"match" : "dismatch");
 		return EXIT_SUCCESS;
+
+	} else if (argc == 4 && strcmp(argv[1], "--cstrcasecmp") == 0) {
+		return test_strutils_cstrcasecmp(argc - 1, argv + 1);
+
 	} else if (argc == 3 && strcmp(argv[1], "--normalize") == 0) {
 		return test_strutils_normalize(argc - 1, argv + 1);
 
-
 	} else if (argc == 3 && strcmp(argv[1], "--strtos64") == 0) {
 		printf("'%s'-->%jd\n", argv[2], strtos64_or_err(argv[2], "strtos64 failed"));
 		return EXIT_SUCCESS;
@@ -1219,11 +1382,16 @@ int main(int argc, char *argv[])
 		printf("'%s'-->%hu\n", argv[2], strtou16_or_err(argv[2], "strtou16 failed"));
 		return EXIT_SUCCESS;
 
+	} else if (argc == 4 && strcmp(argv[1], "--strchr-escaped") == 0) {
+		printf("\"%s\" --> \"%s\"\n", argv[2], ul_strchr_escaped(argv[2], *argv[3]));
+		return EXIT_SUCCESS;
+
 	} else {
 		fprintf(stderr, "usage: %1$s --size <number>[suffix]\n"
 				"       %1$s --cmp-paths <path> <path>\n"
 				"       %1$s --strdup-member <str> <str>\n"
 				"       %1$s --stralnumcmp <str> <str>\n"
+				"       %1$s --cstrcasecmp <str> <str>\n"
 				"       %1$s --normalize <str>\n"
 				"       %1$s --strto{s,u}{16,32,64} <str>\n",
 				argv[0]);
