@@ -111,7 +111,7 @@ static int safe_stat(const char *target, struct stat *st, int nofollow)
 
 	memset(st, 0, sizeof(struct stat));
 
-#ifdef AT_STATX_DONT_SYNC
+#if defined(AT_STATX_DONT_SYNC) && defined (HAVE_STRUCT_STATX)
 	{
 		int rc;
 		struct statx stx = { 0 };
@@ -133,7 +133,8 @@ static int safe_stat(const char *target, struct stat *st, int nofollow)
 			st->st_mode = stx.stx_mode;
 		}
 
-		if (rc == 0 || errno != EOPNOTSUPP)
+		if (rc == 0 ||
+		    (errno != EOPNOTSUPP && errno != ENOSYS && errno != EINVAL))
 			return rc;
 	}
 #endif
